from ngsolve import *
from netgen.geom2d import SplineGeometry
ngsglobals.msg_level=1

#***********************************************************************
def topBottom():
    geometry = SplineGeometry()
    pnts     = [ (0, 0), (1, 0),  (1, 0.5), (1, 1), (0, 1), (0, 0.5)]
    pnums    = [geometry.AppendPoint(*p) for p in pnts]
    lines = [ (pnums[0], pnums[1], "gammaD",  2, 0),
              (pnums[1], pnums[2], "gammaD",  2, 0), 
              (pnums[2], pnums[3], "gammaS",  1, 0),
              (pnums[3], pnums[4], "gammaS",  1, 0),
              (pnums[4], pnums[5], "gammaS",  1, 0),
              (pnums[5], pnums[0], "gammaD",  2, 0),
              (pnums[5], pnums[2], "gammaSD", 1, 2)]
    for p1,p2,bc,left,right in lines:
        geometry.Append( ["line", p1, p2], bc=bc, leftdomain=left, rightdomain=right)

    geometry.SetMaterial(1, "fluid1")
    geometry.SetMaterial(2, "fluid2")
        
    return geometry

def GetInterfaceIndicator(mesh):
    dom_indicator = GridFunction( L2(mesh, order=0) )
    dom_indicator.Set( CoefficientFunction([1,2]) )
  
    ffes            = FacetFESpace(mesh, order=0)
    dom_ind_average = GridFunction(ffes)
    mf  = BilinearForm(ffes, symmetric=True)
    mf += SymbolicBFI(ffes.TrialFunction()*ffes.TestFunction(), element_boundary=True)
    mf.Assemble()
    ff  = LinearForm(ffes)
    ff += SymbolicLFI(dom_indicator*ffes.TestFunction(), element_boundary=True)
    ff.Assemble()
    dom_ind_average.vec.data = mf.mat.Inverse() * ff.vec

    interface_indicator = BitArray(ffes.ndof)
    for i in range(len(dom_ind_average.vec)):
        if (dom_ind_average.vec[i] > 1.75) or (dom_ind_average.vec[i] < 1.25):
            interface_indicator[i] = False
        else:
            interface_indicator[i] = True
    #print(sum(interface_indicator))
    return interface_indicator

mesh = Mesh(topBottom().GenerateMesh (maxh=0.5))

#***********************************************************************
Q1 = FacetFESpace(mesh, order=1, definedon=mesh.Materials("fluid1"))
Q2 = FacetFESpace(mesh, order=1)
#Q2 = FacetFESpace(mesh, order=1, definedon=mesh.Materials("fluid2"))
X    = FESpace([Q1, Q2], dgjumps=True)
p1, p2 = X.TrialFunction()
q1, q2 = X.TestFunction()

#***********************************************************************
a = BilinearForm(X, eliminate_internal=True)
interfaceTerm = 100*p1*q1 + 100*p2*q2
interface_indicator = GetInterfaceIndicator(mesh)
interfacebfi = SymbolicBFI (interfaceTerm, VOL, skeleton=True) 
interfacebfi.SetDefinedOnElements(interface_indicator)
a += interfacebfi
a.Assemble()
print("matrix: ", a.mat.data)
